/* udpshout.h

   Copyright (C) Nicholas Humfrey 2006, Dave Chapman 2002

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   Or, point your browser to http://www.gnu.org/copyleft/gpl.html

*/

#ifndef _UDP_SHOUT_H
#define _UDP_SHOUT_H

#include <inttypes.h>


// Libshout include
#include <shout/shout.h>

#include "mpa_header.h"


// Use by gethostname()
#ifndef HOST_NAME_MAX
#ifdef _POSIX_HOST_NAME_MAX
#define HOST_NAME_MAX	_POSIX_HOST_NAME_MAX
#else
#define HOST_NAME_MAX	(256)
#endif
#endif

#ifndef DOMAIN_NAME_MAX
#define DOMAIN_NAME_MAX	(1024)
#endif



/* Defauls for shout server */
#define SERVER_PORT_DEFAULT			(8000)
#define SERVER_USER_DEFAULT			"source"
#define SERVER_PASSWORD_DEFAULT		"hackme"
#define SERVER_PROTOCOL_DEFAULT		SHOUT_PROTOCOL_HTTP

// The size of MPEG2 TS packets
#define TS_PACKET_SIZE			188

// There seems to be a limit of 32 simultaneous filters in the driver
#define MAX_CHANNEL_COUNT		32

// Maximum allowed PID value
#define MAX_PID_COUNT			8192

// Standard string buffer size
#define STR_BUF_SIZE			1025


/*
	Macros for accessing MPEG-2 TS packet headers
*/
#define TS_PACKET_SYNC_BYTE(b)		(b[0])
#define TS_PACKET_TRANS_ERROR(b)	((b[1]&0x80)>>7)
#define TS_PACKET_PAYLOAD_START(b)	((b[1]&0x40)>>6)
#define TS_PACKET_PRIORITY(b)		((b[1]&0x20)>>4)
#define TS_PACKET_PID(b)			(((b[1]&0x1F)<<8) | b[2])

#define TS_PACKET_SCRAMBLING(b)		((b[3]&0xC0)>>6)
#define TS_PACKET_ADAPTATION(b)		((b[3]&0x30)>>4)
#define TS_PACKET_CONT_COUNT(b)		((b[3]&0x0F)>>0)
#define TS_PACKET_ADAPT_LEN(b)		(b[4])



/*
	Macros for accessing MPEG-2 PES packet headers
*/
#define PES_PACKET_SYNC_BYTE1(b)	(b[0])
#define PES_PACKET_SYNC_BYTE2(b)	(b[1])
#define PES_PACKET_SYNC_BYTE3(b)	(b[2])
#define PES_PACKET_STREAM_ID(b)		(b[3])
#define PES_PACKET_LEN(b)			((b[4] << 8) | b[5])

#define PES_PACKET_SYNC_CODE(b)		((b[6] & 0xC0) >> 6)
#define PES_PACKET_SCRAMBLED(b)		((b[6] & 0x30) >> 4)
#define PES_PACKET_PRIORITY(b)		((b[6] & 0x08) >> 3)
#define PES_PACKET_ALIGNMENT(b)		((b[6] & 0x04) >> 2)
#define PES_PACKET_COPYRIGHT(b)		((b[6] & 0x02) >> 1)
#define PES_PACKET_ORIGINAL(b)		((b[6] & 0x01) >> 0)

#define PES_PACKET_PTS_DTS(b)		((b[7] & 0xC0) >> 6)
#define PES_PACKET_ESCR(b)			((b[7] & 0x20) >> 5)
#define PES_PACKET_ESR(b)			((b[7] & 0x10) >> 4)
#define PES_PACKET_DSM_TRICK(b)		((b[7] & 0x8) >> 3)
#define PES_PACKET_ADD_COPY(b)		((b[7] & 0x4) >> 2)
#define PES_PACKET_CRC(b)			((b[7] & 0x2) >> 1)
#define PES_PACKET_EXTEN(b)			((b[7] & 0x1) >> 0)
#define PES_PACKET_HEAD_LEN(b)		(b[8])

#define PES_PACKET_PTS(b)		((uint32_t)((b[9] & 0x0E) << 29) | \
					 (uint32_t)(b[10] << 22) | \
					 (uint32_t)((b[11] & 0xFE) << 14) | \
					 (uint32_t)(b[12] << 7) | \
					 (uint32_t)(b[13] >> 1))

#define PES_PACKET_DTS(b)		((uint32_t)((b[14] & 0x0E) << 29) | \
					 (uint32_t)(b[15] << 22) | \
					 (uint32_t)((b[16] & 0xFE) << 14) | \
					 (uint32_t)(b[17] << 7) | \
					 (uint32_t)(b[18] >> 1))




/* Structure containing single channel */
typedef struct udpshout_channel_s {

	int num;				// channel number
	int udpsockfd;			// udp socket
	int pid;				// Packet Identifier of audio stream

	char udp_ip[STR_BUF_SIZE];	// input udp ip & port
	int udp_port;
	char *udp_localaddr;	// input udp ip & port
	time_t last_update;		// time, when last packet received.
	
	// Metadata about the channel
	char name[STR_BUF_SIZE];	// channel name
	char genre[STR_BUF_SIZE];	// genre
	char description[STR_BUF_SIZE];	// description
	char url[STR_BUF_SIZE];		// Informational URL


	int continuity_count;	// TS packet continuity counter
	int pes_stream_id;		// PES stream ID
	size_t pes_remaining;	// Number of bytes remaining in current PES packet
	unsigned long pes_ts;	// Timestamp for current PES packet
	
	shout_t *shout;				// libshout structure
	char is_public;				// announce existance?
	char mount[STR_BUF_SIZE];	// mount point

	mpa_header_t mpah;		// Parsed MPEG audio header
	int synced;				// Have MPA sync?
	

	uint8_t *buf;			// MPEG Audio Buffer (with 4 nulls bytes)
	uint32_t buf_size;		// Usable size of MPEG Audio Buffer
	uint32_t buf_used;		// Amount of buffer used
	
	int payload_size;		// Size of the payload
} udpshout_channel_t;


/* Structure server settings */
typedef struct udpshout_server_s {
	char host[STR_BUF_SIZE];
	int port;
	char user[STR_BUF_SIZE];
	char password[STR_BUF_SIZE];
	int protocol;

} udpshout_server_t;


/* In udpshout.c */
extern int channel_count;
extern udpshout_server_t udpshout_server;
extern udpshout_channel_t *channel_map[MAX_PID_COUNT];
extern udpshout_channel_t *channels[MAX_CHANNEL_COUNT];

/* In network.c */
char* gethostname_fqdn();
int input_init_udp(udpshout_channel_t *channel);
void socket_close(int sock);


/* In pes.c */
unsigned char* parse_pes( unsigned char* buf, int size, size_t *payload_size, udpshout_channel_t *chan);


/* In parse_config.c */
int parse_config( char *filepath );



#endif
